--[[ 

	Glass Style for Buffs and Debuffs
	Martin Karer / Sezz, 2008-2012
	http://sezz.at
	
	------------------------------
	
	TODO:
	
		Fix frame anchors:
			[Buffs][( )Weapon Enchants][( )Consolidation Icon]
		
		Reduce SkinButton calls

	------------------------------
	
	CHANGELOG:

		2012-04-21:
			[*] Fixed previousButton

		2012-04-19:
			[*] Raidbuff skinning

		2012-04-17:
			[*] Fixed anchoring when buffs are hidden/consolidated

		2012-04-14:
			[*] Fixed raid buffs tooltip position

		2012-04-14:
			[:] Mists of Pandaria Support [WIP]
			[*] Support for forced buff consolidation (BANANAS!)

		2011-11-24
			[*] Fixed weapon enchant styling (Rogues!)

		2011-08-08
			[*] Fixed thrown weapon enchant gap if no main/offhand enchant is present

		2011-08-08
			[+] Added (rogues') thrown weapon enchant styling
			[-] Removed unsued secure button template code
			[*] TOC bump

		2011-02-15
			[*] Improved buff timers for long durations

		2010-10-26
			[*] Disabled buff consolidation directly

		2010-09-13:
			[:] Cataclysm Update
			[*] Bugfixes

		2010-06-03:
			[*] Again some locals (just testing)...

		2010-05-18:
			[*] Locals...

		2010-02-01:
			[+] New s:UI Scale

		2010-01-16:
			[*] Misc. Bugfixes (hopefully)

		2010-01-05:
			[*] Completely rewritten for 3.3

		2008-??-??:
			[+] First private relase

	------------------------------------------------------------

	[*] Fix  [+] Added [-] Removed  [>] TODO
	[?] WTH? [:] Note  [!] Important/Warning

--]]


local _G = getfenv(0);
local dummyFunction = function() return; end;
local DebuffTypeColor = DebuffTypeColor;
local UnitDebuff = UnitDebuff;
local GetWeaponEnchantInfo = GetWeaponEnchantInfo;
local ChatFrame_TimeBreakDown = ChatFrame_TimeBreakDown;


-- Disable buff consolidation
if (GetCVar("consolidateBuffs")) then
	SetCVar("consolidateBuffs", 0);
	consolidatedBuffsEnabled = false;
else
	consolidatedBuffsEnabled = true;
end
SetCVar("buffDurations", 1);


-- Frame Positions
local buffFrameY = sCore:ScaleUp(36);
local debuffFrameY = buffFrameY + (TemporaryEnchantFrame:GetHeight() * 3);


-- Considolated Buffs ("Raid" Buffs)
if (consolidatedBuffsEnabled) then
	ConsolidatedBuffsIcon:SetTexture("INTERFACE\\ICONS\\inv_misc_food_23");
	ConsolidatedBuffsIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	ConsolidatedBuffsIcon:SetSize(30, 30);
	ConsolidatedBuffs:SetScale(0.76);
	ConsolidatedBuffsCount:SetFont([[Fonts\FRIZQT__.ttf]], 16, "OUTLINE")
	ConsolidatedBuffsCount:SetJustifyH("RIGHT")
	sCore:Glassify(ConsolidatedBuffs, {1, 0.9, 0}, 3);
	
	local ConsolidatedBuffsTooltip_FixAnchor = function()
		ConsolidatedBuffsTooltip:ClearAllPoints();
		ConsolidatedBuffsTooltip:SetPoint("BOTTOM", ConsolidatedBuffs, "TOP");
	end
	
	hooksecurefunc(ConsolidatedBuffsTooltip, "Show", ConsolidatedBuffsTooltip_FixAnchor);
	
	ConsolidatedBuffsTooltip:SetScale(0.78);
	for i = 1, NUM_LE_RAID_BUFF_TYPES do
		local raidBuff = _G["ConsolidatedBuffsTooltipBuff"..i];
		local raidBuffIcon = _G["ConsolidatedBuffsTooltipBuff"..i.."Icon"];
		sCore:ApplyBackdropBorder(raidBuff)
		raidBuffIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
		raidBuffIcon:SetSize(15,15);
		raidBuffIcon:SetPoint("LEFT", 1, 0);
	end
end


-- Moving...
BuffFrame:SetParent(UIParent);
BuffFrame:SetHeight(TemporaryEnchantFrame:GetHeight())
BuffFrame:ClearAllPoints();
BuffFrame:SetPoint("BOTTOMRIGHT", Minimap, "TOPRIGHT", -1, buffFrameY);
BuffFrame.SetPoint = dummyFunction

local UpdateTemporaryEnchantFrameAnchors = function()
	if (ConsolidatedBuffs:IsShown()) then
		TemporaryEnchantFrame:ClearAllPoints();
		TemporaryEnchantFrame:SetPoint("TOPRIGHT", ConsolidatedBuffs, "TOPLEFT", -17.5, 0)
	else
		TemporaryEnchantFrame:ClearAllPoints();
		TemporaryEnchantFrame:SetPoint("BOTTOMRIGHT", Minimap, "TOPRIGHT", -1, buffFrameY);
	end
	
--	BuffFrame_UpdateAllBuffAnchors();
end

hooksecurefunc(ConsolidatedBuffs, "Show", UpdateTemporaryEnchantFrameAnchors);
hooksecurefunc(ConsolidatedBuffs, "Hide", UpdateTemporaryEnchantFrameAnchors);

TemporaryEnchantFrame:SetParent(UIParent);
UpdateTemporaryEnchantFrameAnchors();


-- Skinning...
local GetPreviousVisibleBuff = function(index)
	local previousButton;

	for i = index - 1, 1, -1 do
		local button = _G["BuffButton"..i];
		if (button and button:IsVisible()) then
			previousButton = button;
			break;
		end
	end
	
	return previousButton;
end

local SkinButton = function(buttonName, index, isDebuff)
	local auraButton	= _G[buttonName..index];
	local auraGlass		= _G[buttonName..index.."Glass"];
	local auraDuration	= _G[buttonName..index.."Duration"];
--	local auraIcon		= _G[buttonName..index.."Icon"];
	local auraBorder	= _G[buttonName..index.."Border"];
	
	if (auraButton and auraButton:IsShown()) then
		if (not auraGlass) then
			-- Duration
			if auraDuration then
				auraDuration:ClearAllPoints();
				auraDuration:SetPoint("TOP", auraButton, "TOP", 0, 17);
				auraDuration:SetFont([[Fonts\FRIZQT__.ttf]], 12)
			end

			-- Resize
			auraButton:SetScale(0.76);

			-- Glass
			if (auraBorder) then auraBorder:Hide();	end
			if (isDebuff == false) then
				sCore:Glassify(auraButton, {1, 0, 1}, 3);
			elseif (isDebuff == nil) then
				sCore:Glassify(auraButton, nil, 3);
			end
		end
		
		if (isDebuff == true) then
			-- Debuff Border Color
			local _, _, _, _, debuffType = UnitDebuff("player", index);
			local debuffColor = DebuffTypeColor[debuffType] or DebuffTypeColor["none"]; 

			sCore:Glassify(auraButton, {debuffColor["r"], debuffColor["g"], debuffColor["b"]});
		end
	end
	
	-- Fix Anchoring
	if (auraButton) then
		if (buttonName ~= "TempEnchant" and auraButton.parent ~= BuffFrame) then
			auraButton:SetParent(BuffFrame);
			auraButton.parent = BuffFrame;
		end
		
		-- Fix consolidation gap
		-- Other idea: move invisible buffs under last visible?
		local previousButton, indexDisplay;
		if (consolidatedBuffsEnabled and buttonName == "BuffButton") then
			previousButton = GetPreviousVisibleBuff(index);
			indexDisplay = index;
			if (previousButton) then
				indexDisplay = previousButton:GetID() + 1;
			else
				indexDisplay = 1;
			end
		else
			indexDisplay = index;
			if (indexDisplay > 1) then
				previousButton = _G[buttonName..(index - 1)];
			end
		end
		
		if (indexDisplay == 1) then
			if (isDebuff) then
				-- Debuff
				auraButton:ClearAllPoints();
				auraButton:SetPoint("BOTTOMRIGHT", Minimap, "TOPRIGHT", -1, debuffFrameY);
			elseif (buttonName ~= "TempEnchant") then
				-- Buff
				auraButton:ClearAllPoints();
				local hasMainHandEnchant, _, _, hasOffHandEnchant, _, _, hasThrownEnchant = GetWeaponEnchantInfo()
				if (hasMainHandEnchant or hasOffHandEnchant or hasThrownEnchant) then
					auraButton:SetPoint("TOPRIGHT", _G["TemporaryEnchantFrame"], "TOPLEFT", -8, 0);
				else
					auraButton:SetPoint("TOPRIGHT", _G["BuffFrame"], "TOPRIGHT", ConsolidatedBuffs:IsShown() and (-16.5 - ConsolidatedBuffs:GetWidth())  or 0, 0);
				end
			end
		else
			auraButton:ClearAllPoints();
			auraButton:SetPoint("TOPRIGHT", previousButton, "TOPLEFT", -10, 0);
		end
	end
end


hooksecurefunc("BuffFrame_UpdateAllBuffAnchors", function()
	-- Weapon Enchants
	local hasMainHandEnchant, _, _, hasOffHandEnchant, _, _, hasThrownEnchant = GetWeaponEnchantInfo()
	SkinButton("TempEnchant", 1, false);
	SkinButton("TempEnchant", 2, false);
	SkinButton("TempEnchant", 3, false);

	-- Buffs
	for i = 1, BUFF_ACTUAL_DISPLAY do
		SkinButton("BuffButton", i);
	end
	
	-- Debuffs
	for i = 1, DEBUFF_ACTUAL_DISPLAY do
		SkinButton("DebuffButton", i, true);
	end
end)


hooksecurefunc("DebuffButton_UpdateAnchors", function(buttonName, index)
	SkinButton(buttonName, index, true);
end)


-- Telo's BuffTimers [FAN UPDATE] by vlaad78
-- http://www.wowinterface.com/downloads/info13140-TelosBuffTimersFANUPDATE.html
local BT_SetTimeText = function(button, time)
	if (time <= 0) then
		button:SetText("");
	elseif (time < 600) then
		local d, h, m, s = ChatFrame_TimeBreakDown(time);
		button:SetFormattedText("%1d:%02d", m, s);
	elseif (time < 3600) then
		local d, h, m, s = ChatFrame_TimeBreakDown(time);
		button:SetFormattedText("%02d:%02d", m, s);
	else
		local d, h, m, s = ChatFrame_TimeBreakDown(time);
		button:SetFormattedText("%1dh:%02d", h, m);
	end
end


local BT_AuraButton_UpdateDuration = function(buffButton, timeLeft)
	if (SHOW_BUFF_DURATIONS == "1") then
		local duration = _G[buffButton:GetName().."Duration"];
		if (timeLeft) then
			BT_SetTimeText(duration, timeLeft);
			duration:Show();
		else
			duration:Hide();
		end
	end
end


hooksecurefunc("AuraButton_UpdateDuration", BT_AuraButton_UpdateDuration);

